// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/common/GraphBase.hpp>

size_t GraphBase::getNVertices() const
{
    return firstVertexIncidences.size();
}

size_t GraphBase::getNEdges() const
{
    return nextIncidences.size() >> 1;
}

bool GraphBase::areAdjacent(const int vertex1, const int vertex2) const
{
    auto it = begin(vertex1);
    while (it != end())
    {
        if (it->vertex == vertex2)
        {
            return true;
        }
        it++;
    }
    return false;
}

GraphBase::NeighborIterator GraphBase::begin(const int vertex) const
{
    return { *this, vertex };
}

GraphBase::NeighborIterator GraphBase::end() const
{
    return { *this, -1 };
}

int GraphBase::addVertexInternal()
{
    firstVertexIncidences.push_back(-1);
    return static_cast<int>(firstVertexIncidences.size()) - 1;
}

void GraphBase::addEdgeInternal(const int vertex1, const int vertex2)
{
    incidenceVertices.push_back(vertex2);
    nextIncidences.push_back(firstVertexIncidences[vertex1]);
    firstVertexIncidences[vertex1] = static_cast<int>(incidenceVertices.size()) - 1;

    incidenceVertices.push_back(vertex1);
    nextIncidences.push_back(firstVertexIncidences[vertex2]);
    firstVertexIncidences[vertex2] = static_cast<int>(incidenceVertices.size()) - 1;
}
